#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <armarx/navigation/client/Navigator.h>


armarx::navigation::client::Navigator::Navigator(const InjectedServices& services) : srv{services}
{
    ARMARX_CHECK_NOT_NULL(srv.navigator) << "Navigator service must not be null!";
    ARMARX_CHECK_NOT_NULL(srv.subscriber) << "Subscriber service must not be null!";

    auto stopped_callback = [this](const auto& e) { stopped(StopEvent(e)); };

    srv.subscriber->onGoalReached(stopped_callback);
    srv.subscriber->onSafetyStopTriggered(stopped_callback);
    srv.subscriber->onUserAbortTriggered(stopped_callback);
    srv.subscriber->onInternalError(stopped_callback);
}


void
armarx::navigation::client::Navigator::moveTo(const core::Pose& pose, core::NavigationFrame frame)
{
    moveTo(std::vector<core::Pose>{pose}, frame);
}


void
armarx::navigation::client::Navigator::moveTo(const std::vector<core::Pose>& waypoints,
                                              core::NavigationFrame frame)
{
    srv.navigator->moveTo(waypoints, frame);
}


void
armarx::navigation::client::Navigator::moveTowards(const core::Direction& direction,
                                                   core::NavigationFrame frame)
{
    srv.navigator->moveTowards(direction, frame);
}


void
armarx::navigation::client::Navigator::pause()
{
    srv.navigator->pause();
}


void
armarx::navigation::client::Navigator::resume()
{
    srv.navigator->resume();
}


void
armarx::navigation::client::Navigator::stop()
{
    srv.navigator->stop();
}


void
armarx::navigation::client::Navigator::onGoalReached(const std::function<void(void)>& callback)
{
    onGoalReached([&callback](const core::GoalReachedEvent&) { callback(); });
}


void
armarx::navigation::client::Navigator::onGoalReached(
    const std::function<void(const core::GoalReachedEvent&)>& callback)
{
}


void
armarx::navigation::client::Navigator::onWaypointReached(const std::function<void(int)>& callback)
{
}


armarx::navigation::client::StopEvent
armarx::navigation::client::Navigator::waitForStop()
{
    std::unique_lock l{stoppedInfo.m};
    stoppedInfo.cv.wait(l, [&i = stoppedInfo] { return i.event.has_value(); });

    StopEvent e = stoppedInfo.event.value();
    stoppedInfo.event.reset();
    return e;
}


void
armarx::navigation::client::Navigator::stopped(const StopEvent& e)
{
    {
        std::scoped_lock l{stoppedInfo.m};
        stoppedInfo.event = e;
    }
    stoppedInfo.cv.notify_all();
}
