#include <armarx/navigation/client/services/IceNavigator.h>

std::vector<Eigen::Matrix4f>
convert(const std::vector<armarx::navigation::core::Pose>& ps)
{
    using namespace armarx::navigation;
    std::vector<Eigen::Matrix4f> ms;
    ms.reserve(ps.size());
    std::transform(
        ps.begin(), ps.end(), std::back_inserter(ms), [](const auto& p) { return p.matrix(); });
    return ms;
}

armarx::navigation::client::IceNavigator::IceNavigator()
{
    // pass
}

armarx::navigation::client::IceNavigator::~IceNavigator()
{
    // pass
}

void
armarx::navigation::client::IceNavigator::setNavigatorComponent(
    const NavigatorInterfacePrx& navigator)
{
    this->navigator = navigator;
}

void
armarx::navigation::client::IceNavigator::createConfig(const client::NavigationStackConfig& config,
                                                       const std::string& configId)
{
    this->configId = configId;
    navigator->createConfig(config.toAron(), configId);
}

void
armarx::navigation::client::IceNavigator::moveTo(const std::vector<core::Pose>& waypoints,
                                                 core::NavigationFrame navigationFrame)
{
    navigator->moveTo(
        convert(waypoints), core::NavigationFrameNames.to_name(navigationFrame), configId);
}

void
armarx::navigation::client::IceNavigator::moveTowards(const core::Direction& direction,
                                                      core::NavigationFrame navigationFrame)
{
    navigator->moveTowards(
        direction, core::NavigationFrameNames.to_name(navigationFrame), configId);
}

void
armarx::navigation::client::IceNavigator::pause()
{
    navigator->pause(configId);
}

void
armarx::navigation::client::IceNavigator::resume()
{
    navigator->resume(configId);
}

void
armarx::navigation::client::IceNavigator::stop()
{
    navigator->stop(configId);
}

bool
armarx::navigation::client::IceNavigator::isPaused() const noexcept
{
    return navigator->isPaused(configId);
}

bool
armarx::navigation::client::IceNavigator::isStopped() const noexcept
{
    return navigator->isStopped(configId);
}
