/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/aron_conversions.h>
#include <RobotAPI/libraries/armem/core/forward_declarations.h>

#include <SemanticObjectRelations/RelationGraph/RelationGraph.h>
#include <armarx/navigation/core/types.h>
#include <armarx/navigation/graph/aron/Graph.aron.generated.h>


namespace armarx::navigation::graph
{

    struct VertexAttribs : public semrel::ShapeVertex
    {
        armarx::navigation::graph::arondto::Vertex aron;

        std::string getName() const;

        bool hasPose() const;
        core::Pose getPose() const;
        void setPose(const core::Pose& pose);

    private:
        std::optional<core::Pose> _pose;
    };

    struct EdgeAttribs
    {
        armarx::navigation::graph::arondto::Edge aron;
    };

    struct GraphAttribs
    {
    };

    using Graph = semrel::RelationGraph<VertexAttribs, EdgeAttribs, GraphAttribs>;


    void toAron(arondto::Graph& dto, const Graph& bo);
    void fromAron(const arondto::Graph& dto, Graph& bo);


    // Location resolution

    void resolveLocation(Graph::Vertex& vertex,
                         const aron::datanavigator::DictNavigatorPtr& locationData);


    template <class MemoryContainerT>
    bool
    resolveLocation(Graph::Vertex& vertex, const MemoryContainerT& locationContainer)
    {
        armem::MemoryID locationID;
        fromAron(vertex.attrib().aron.locationID, locationID);

        if (const auto* instance = locationContainer.findLatestInstance(locationID))
        {
            resolveLocation(vertex, instance->data());
            return true;
        }
        else
        {
            return false;
        }
    }


    template <class MemoryContainerT>
    void
    resolveLocations(Graph& graph, const MemoryContainerT& locationContainer)
    {
        for (graph::Graph::Vertex vertex : graph.vertices())
        {
            resolveLocation(vertex, locationContainer);
        }
    }

} // namespace armarx::navigation::graph
