#include "Writer.h"

#include <RobotAPI/libraries/aron/core/navigator/data/AllNavigators.h>

#include <armarx/navigation/core/constants.h>


namespace armarx::navigation::mem::client::param
{
    bool
    Writer::store(
        const std::unordered_map<core::StackLayer, aron::datanavigator::DictNavigatorPtr>& stack,
        const std::string& clientID,
        const core::TimestampUs& timestamp)
    {
        ARMARX_CHECK(not stack.empty());

        armem::Time ts = armem::Time::microSeconds(timestamp.count());

        ARMARX_INFO << "timestamp chrono " << timestamp;
        ARMARX_INFO << "timestamp " << ts;

        armem::Commit commit;

        for (const auto& [layer, data] : stack)
        {
            const std::string layerName = core::StackLayerNames.to_name(layer);

            armem::EntityUpdate update;
            update.entityID = armem::MemoryID()
                                  .withMemoryName(properties().memoryName)
                                  .withCoreSegmentName(properties().coreSegmentName)
                                  .withProviderSegmentName(clientID)
                                  .withEntityName(layerName)
                                  .withTimestamp(ts);
            update.timeCreated = ts;
            update.instancesData = {data};
            commit.add(update);
        }

        std::lock_guard g{memoryWriterMutex()};
        armem::CommitResult updateResult = memoryWriter().commit(commit);

        ARMARX_DEBUG << updateResult;

        if (not updateResult.allSuccess())
        {
            ARMARX_ERROR << updateResult.allErrorMessages();
        }
        return updateResult.allSuccess();
    }

    std::string
    Writer::propertyPrefix() const
    {
        return "mem.nav.param.";
    }

    Writer::Properties
    Writer::defaultProperties() const
    {
        return Properties{
            .memoryName = "Navigation",
            .coreSegmentName = "Parameterization",
            .providerName = "" // clientId
        };
    }

} // namespace armarx::navigation::mem::client::param
